@@ -43,7 +43,7 @@ _addr_family_to_path_part(int addr_family)
 
 #define NM_TYPE_DHCP_DHCLIENT (nm_dhcp_dhclient_get_type())
 #define NM_DHCP_DHCLIENT(obj) \
-    (G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclient))
+    (_NM_G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclient))
 #define NM_DHCP_DHCLIENT_CLASS(klass) \
     (G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclientClass))
 #define NM_IS_DHCP_DHCLIENT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), NM_TYPE_DHCP_DHCLIENT))
@@ -59,10 +59,10 @@ static GType nm_dhcp_dhclient_get_type(void);
 /*****************************************************************************/
 
 typedef struct {
-    char *          conf_file;
-    const char *    def_leasefile;
-    char *          lease_file;
-    char *          pid_file;
+    char           *conf_file;
+    const char     *def_leasefile;
+    char           *lease_file;
+    char           *pid_file;
     NMDhcpListener *dhcp_listener;
 } NMDhcpDhclientPrivate;
 
@@ -82,6 +82,10 @@ G_DEFINE_TYPE(NMDhcpDhclient, nm_dhcp_dhclient, NM_TYPE_DHCP_CLIENT)
 
 /*****************************************************************************/
 
+static GBytes *read_duid_from_lease(NMDhcpDhclient *self);
+
+/*****************************************************************************/
+
 static const char *
 nm_dhcp_dhclient_get_path(void)
 {
@@ -105,7 +109,7 @@ static char *
 get_dhclient_leasefile(int         addr_family,
                        const char *iface,
                        const char *uuid,
-                       char **     out_preferred_path)
+                       char      **out_preferred_path)
 {
     gs_free char *path = NULL;
 
@@ -145,21 +149,21 @@ get_dhclient_leasefile(int         addr_family,
 }
 
 static gboolean
-merge_dhclient_config(NMDhcpDhclient *    self,
+merge_dhclient_config(NMDhcpDhclient     *self,
                       int                 addr_family,
-                      const char *        iface,
-                      const char *        conf_file,
-                      GBytes *            client_id,
-                      const char *        anycast_address,
-                      const char *        hostname,
+                      const char         *iface,
+                      const char         *conf_file,
+                      GBytes             *client_id,
+                      const char         *anycast_address,
+                      const char         *hostname,
                       guint32             timeout,
                       gboolean            use_fqdn,
                       NMDhcpHostnameFlags hostname_flags,
-                      const char *        mud_url,
-                      const char *const * reject_servers,
-                      const char *        orig_path,
-                      GBytes **           out_new_client_id,
-                      GError **           error)
+                      const char         *mud_url,
+                      const char *const  *reject_servers,
+                      const char         *orig_path,
+                      GBytes            **out_new_client_id,
+                      GError            **error)
 {
     gs_free char *orig = NULL;
     gs_free char *new  = NULL;
@@ -274,19 +278,19 @@ find_existing_config(NMDhcpDhclient *self, int addr_family, const char *iface, c
  * config file along with the NM options.
  */
 static char *
-create_dhclient_config(NMDhcpDhclient *    self,
+create_dhclient_config(NMDhcpDhclient     *self,
                        int                 addr_family,
-                       const char *        iface,
-                       const char *        uuid,
-                       GBytes *            client_id,
-                       const char *        anycast_address,
-                       const char *        hostname,
+                       const char         *iface,
+                       const char         *uuid,
+                       GBytes             *client_id,
+                       const char         *anycast_address,
+                       const char         *hostname,
                        guint32             timeout,
                        gboolean            use_fqdn,
                        NMDhcpHostnameFlags hostname_flags,
-                       const char *        mud_url,
-                       const char *const * reject_servers,
-                       GBytes **           out_new_client_id)
+                       const char         *mud_url,
+                       const char *const  *reject_servers,
+                       GBytes            **out_new_client_id)
 {
     gs_free char *orig = NULL;
     char *new          = NULL;
@@ -330,27 +334,29 @@ create_dhclient_config(NMDhcpDhclient *    self,
 
 static gboolean
 dhclient_start(NMDhcpClient *client,
-               const char *  mode_opt,
+               gboolean      set_mode,
                gboolean      release,
-               pid_t *       out_pid,
-               GError **     error)
+               gboolean      set_duid,
+               pid_t        *out_pid,
+               GError      **error)
 {
-    NMDhcpDhclient *       self       = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv       = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    NMDhcpDhclient              *self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate       *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     gs_unref_ptrarray GPtrArray *argv = NULL;
     pid_t                        pid;
-    gs_free_error GError *    local = NULL;
-    const char *              iface;
-    const char *              uuid;
-    const char *              system_bus_address;
-    const char *              dhclient_path;
-    char *                    binary_name;
-    gs_free char *            cmd_str                  = NULL;
-    gs_free char *            pid_file                 = NULL;
-    gs_free char *            system_bus_address_env   = NULL;
-    gs_free char *            preferred_leasefile_path = NULL;
-    int                       addr_family;
-    const NMDhcpClientConfig *client_config;
+    gs_free_error GError        *local = NULL;
+    const char                  *iface;
+    const char                  *uuid;
+    const char                  *system_bus_address;
+    const char                  *dhclient_path;
+    char                        *binary_name;
+    gs_free char                *cmd_str                  = NULL;
+    gs_free char                *pid_file                 = NULL;
+    gs_free char                *system_bus_address_env   = NULL;
+    gs_free char                *preferred_leasefile_path = NULL;
+    int                          addr_family;
+    const NMDhcpClientConfig    *client_config;
+    char                         pd_length_str[16];
 
     g_return_val_if_fail(!priv->pid_file, FALSE);
     client_config = nm_dhcp_client_get_config(client);
@@ -410,8 +416,11 @@ dhclient_start(NMDhcpClient *client,
     }
 
     /* Save the DUID to the leasefile dhclient will actually use */
-    if (addr_family == AF_INET6) {
-        if (!nm_dhcp_dhclient_save_duid(priv->lease_file, client_config->client_id, &local)) {
+    if (set_duid && addr_family == AF_INET6) {
+        if (!nm_dhcp_dhclient_save_duid(priv->lease_file,
+                                        nm_dhcp_client_get_effective_client_id(client),
+                                        client_config->v6.enforce_duid,
+                                        &local)) {
             nm_utils_error_set(error,
                                NM_UTILS_ERROR_UNKNOWN,
                                "failed to save DUID to '%s': %s",
@@ -439,17 +448,33 @@ dhclient_start(NMDhcpClient *client,
     }
 
     if (addr_family == AF_INET6) {
-        guint prefixes = client_config->v6.needed_prefixes;
+        guint       prefixes = client_config->v6.needed_prefixes;
+        const char *mode_opt;
 
         g_ptr_array_add(argv, (gpointer) "-6");
 
-        if (prefixes > 0 && nm_streq0(mode_opt, "-S")) {
-            /* -S is incompatible with -P, only use the latter */
+        if (!set_mode)
+            mode_opt = NULL;
+        else if (!client_config->v6.info_only)
+            mode_opt = "-N";
+        else if (prefixes == 0)
+            mode_opt = "-S";
+        else
             mode_opt = NULL;
-        }
 
         if (mode_opt)
             g_ptr_array_add(argv, (gpointer) mode_opt);
+
+        if (prefixes > 0 && client_config->v6.pd_hint_length > 0) {
+            if (!IN6_IS_ADDR_UNSPECIFIED(&client_config->v6.pd_hint_addr)) {
+                _LOGW("dhclient only supports a length as prefix delegation hint, not a prefix");
+            }
+
+            nm_sprintf_buf(pd_length_str, "%u", client_config->v6.pd_hint_length);
+            g_ptr_array_add(argv, "--prefix-len-hint");
+            g_ptr_array_add(argv, pd_length_str);
+        }
+
         while (prefixes--)
             g_ptr_array_add(argv, (gpointer) "-P");
     }
@@ -515,9 +540,9 @@ dhclient_start(NMDhcpClient *client,
 static gboolean
 ip4_start(NMDhcpClient *client, GError **error)
 {
-    NMDhcpDhclient *       self             = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv             = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
-    gs_unref_bytes GBytes *   new_client_id = NULL;
+    NMDhcpDhclient           *self          = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate    *priv          = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    gs_unref_bytes GBytes    *new_client_id = NULL;
     const NMDhcpClientConfig *client_config;
 
     client_config = nm_dhcp_client_get_config(client);
@@ -542,19 +567,20 @@ ip4_start(NMDhcpClient *client, GError **error)
         return FALSE;
     }
 
-    if (new_client_id) {
-        nm_assert(!client_config->client_id);
-        nm_dhcp_client_set_effective_client_id(client, new_client_id);
-    }
-    return dhclient_start(client, NULL, FALSE, NULL, error);
+    /* Note that the effective-client-id for IPv4 here might be unknown/NULL. */
+    nm_assert(!new_client_id || !client_config->client_id);
+    nm_dhcp_client_set_effective_client_id(client, client_config->client_id ?: new_client_id);
+
+    return dhclient_start(client, FALSE, FALSE, FALSE, NULL, error);
 }
 
 static gboolean
 ip6_start(NMDhcpClient *client, const struct in6_addr *ll_addr, GError **error)
 {
-    NMDhcpDhclient *          self = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *   priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    NMDhcpDhclient           *self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate    *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     const NMDhcpClientConfig *config;
+    gs_unref_bytes GBytes    *effective_client_id = NULL;
 
     config = nm_dhcp_client_get_config(client);
 
@@ -581,13 +607,18 @@ ip6_start(NMDhcpClient *client, const struct in6_addr *ll_addr, GError **error)
         return FALSE;
     }
 
-    return dhclient_start(client, config->v6.needed_prefixes ? "-S" : "-N", FALSE, NULL, error);
+    nm_assert(config->client_id);
+    if (!config->v6.enforce_duid)
+        effective_client_id = read_duid_from_lease(self);
+    nm_dhcp_client_set_effective_client_id(client, effective_client_id ?: config->client_id);
+
+    return dhclient_start(client, TRUE, FALSE, TRUE, NULL, error);
 }
 
 static void
 stop(NMDhcpClient *client, gboolean release)
 {
-    NMDhcpDhclient *       self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclient        *self = NM_DHCP_DHCLIENT(client);
     NMDhcpDhclientPrivate *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     int                    errsv;
 
@@ -615,7 +646,7 @@ stop(NMDhcpClient *client, gboolean release)
     if (release) {
         pid_t rpid = -1;
 
-        if (dhclient_start(client, NULL, TRUE, &rpid, NULL)) {
+        if (dhclient_start(client, FALSE, TRUE, FALSE, &rpid, NULL)) {
             /* Wait a few seconds for the release to happen */
             nm_dhcp_client_stop_pid(rpid, nm_dhcp_client_get_iface(client));
         }
@@ -623,14 +654,14 @@ stop(NMDhcpClient *client, gboolean release)
 }
 
 static GBytes *
-get_duid(NMDhcpClient *client)
+read_duid_from_lease(NMDhcpDhclient *self)
 {
-    NMDhcpDhclient *          self = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *   priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    NMDhcpClient             *client = NM_DHCP_CLIENT(self);
+    NMDhcpDhclientPrivate    *priv   = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     const NMDhcpClientConfig *client_config;
-    GBytes *                  duid      = NULL;
-    gs_free char *            leasefile = NULL;
-    GError *                  error     = NULL;
+    GBytes                   *duid      = NULL;
+    gs_free char             *leasefile = NULL;
+    GError                   *error     = NULL;
 
     client_config = nm_dhcp_client_get_config(client);
 
@@ -712,14 +743,13 @@ static void
 nm_dhcp_dhclient_class_init(NMDhcpDhclientClass *dhclient_class)
 {
     NMDhcpClientClass *client_class = NM_DHCP_CLIENT_CLASS(dhclient_class);
-    GObjectClass *     object_class = G_OBJECT_CLASS(dhclient_class);
+    GObjectClass      *object_class = G_OBJECT_CLASS(dhclient_class);
 
     object_class->dispose = dispose;
 
     client_class->ip4_start = ip4_start;
     client_class->ip6_start = ip6_start;
     client_class->stop      = stop;
-    client_class->get_duid  = get_duid;
 }
 
 const NMDhcpClientFactory _nm_dhcp_client_factory_dhclient = {
