@@ -703,11 +703,14 @@ class Util:
         return typ(pexp, valgrind_log)
 
     @staticmethod
-    def cmd_call_pexpect_nmcli(args):
+    def cmd_call_pexpect_nmcli(args, extra_env={}):
+        extra_env = extra_env.copy()
+        extra_env.update({"NO_COLOR": "1"})
+
         return Util.cmd_call_pexpect(
             ENV_NM_TEST_CLIENT_NMCLI_PATH,
             args,
-            {"NO_COLOR": "1"},
+            extra_env,
         )
 
 
@@ -2158,6 +2161,55 @@ class TestNmcli(unittest.TestCase):
         nmc.pexp.expect(pexpect.EOF)
         Util.valgrind_check_log(nmc.valgrind_log, "test_ask_mode")
 
+    @Util.skip_without_pexpect
+    @nm_test
+    def test_ask_offline(self):
+        # Make sure we're not using D-Bus
+        no_dbus_env = {
+            "DBUS_SYSTEM_BUS_ADDRESS": "very:invalid",
+            "DBUS_SESSION_BUS_ADDRESS": "very:invalid",
+        }
+
+        nmc = Util.cmd_call_pexpect_nmcli(
+            ["--offline", "--ask", "c", "add"], extra_env=no_dbus_env
+        )
+        nmc.pexp.expect("Connection type:")
+        nmc.pexp.sendline("ethernet")
+        nmc.pexp.expect("Interface name:")
+        nmc.pexp.sendline("eth0")
+        nmc.pexp.expect("There are 3 optional settings for Wired Ethernet.")
+        nmc.pexp.expect("Do you want to provide them\? \(yes/no\) \[yes]")
+        nmc.pexp.sendline("no")
+        nmc.pexp.expect("There are 2 optional settings for IPv4 protocol.")
+        nmc.pexp.expect("Do you want to provide them\? \(yes/no\) \[yes]")
+        nmc.pexp.sendline("no")
+        nmc.pexp.expect("There are 2 optional settings for IPv6 protocol.")
+        nmc.pexp.expect("Do you want to provide them\? \(yes/no\) \[yes]")
+        nmc.pexp.sendline("no")
+        nmc.pexp.expect("There are 4 optional settings for Proxy.")
+        nmc.pexp.expect("Do you want to provide them\? \(yes/no\) \[yes]")
+        nmc.pexp.sendline("no")
+        nmc.pexp.expect(
+            "\[connection\]\r\n"
+            + "id=ethernet\r\n"
+            + "uuid=.*\r\n"
+            + "type=ethernet\r\n"
+            + "interface-name=eth0\r\n"
+            + "\r\n"
+            + "\[ethernet\]\r\n"
+            + "\r\n"
+            + "\[ipv4\]\r\n"
+            + "method=auto\r\n"
+            + "\r\n"
+            + "\[ipv6\]\r\n"
+            + "addr-gen-mode=default\r\n"
+            + "method=auto\r\n"
+            + "\r\n"
+            + "\[proxy\]\r\n"
+        )
+        nmc.pexp.expect(pexpect.EOF)
+        Util.valgrind_check_log(nmc.valgrind_log, "test_ask_offline")
+
     @Util.skip_without_pexpect
     @nm_test
     def test_monitor(self):
@@ -2203,8 +2255,8 @@ class TestNmCloudSetup(unittest.TestCase):
         Util.skip_without_NM()
         self.ctx = NMTestContext(self._testMethodName)
 
-    _mac1 = "9e:c0:3e:92:24:2d"
-    _mac2 = "53:e9:7e:52:8d:a8"
+    _mac1 = "cc:00:00:00:00:01"
+    _mac2 = "cc:00:00:00:00:02"
 
     _ip1 = "172.31.26.249"
     _ip2 = "172.31.176.249"
@@ -2271,7 +2323,7 @@ class TestNmCloudSetup(unittest.TestCase):
 
     def _mock_devices(self):
         # Add a device with an active connection that has IPv4 configured
-        self.ctx.srv.op_AddObj("WiredDevice", iface="eth0", mac="9e:c0:3e:92:24:2d")
+        self.ctx.srv.op_AddObj("WiredDevice", iface="eth0", mac="cc:00:00:00:00:01")
         self.ctx.srv.addAndActivateConnection(
             {
                 "connection": {"type": "802-3-ethernet", "id": "con-eth0"},
@@ -2282,7 +2334,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         # The second connection has no IPv4
-        self.ctx.srv.op_AddObj("WiredDevice", iface="eth1", mac="53:e9:7e:52:8d:a8")
+        self.ctx.srv.op_AddObj("WiredDevice", iface="eth1", mac="cc:00:00:00:00:02")
         self.ctx.srv.addAndActivateConnection(
             {"connection": {"type": "802-3-ethernet", "id": "con-eth1"}},
             "/org/freedesktop/NetworkManager/Devices/2",
@@ -2351,7 +2403,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider aliyun detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: start fetching meta data")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
@@ -2373,7 +2425,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider aliyun detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: starting")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
@@ -2430,7 +2482,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider azure detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("found azure interfaces: 2")
         nmc.pexp.expect("interface\[0]: found a matching device with hwaddr")
         nmc.pexp.expect(
@@ -2459,7 +2511,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider azure detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: starting")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
@@ -2506,7 +2558,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider ec2 detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: starting")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
@@ -2528,7 +2580,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider ec2 detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: starting")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
@@ -2566,7 +2618,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider GCP detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("found GCP interfaces: 2")
         nmc.pexp.expect("GCP interface\[0]: found a requested device with hwaddr")
         nmc.pexp.expect("get-config: success")
@@ -2589,7 +2641,7 @@ class TestNmCloudSetup(unittest.TestCase):
         )
 
         nmc.pexp.expect("provider GCP detected")
-        nmc.pexp.expect("found interfaces: 9E:C0:3E:92:24:2D, 53:E9:7E:52:8D:A8")
+        nmc.pexp.expect("found interfaces: CC:00:00:00:00:01, CC:00:00:00:00:02")
         nmc.pexp.expect("get-config: starting")
         nmc.pexp.expect("get-config: success")
         nmc.pexp.expect("meta data received")
